@@ -1,13 +1,10 @@
 //! Wrappers representing concrete btf types.
 
-use std::ffi::CStr;
+use std::ffi::OsStr;
 use std::fmt;
 use std::fmt::Display;
 use std::ops::Deref;
 
-use num_enum::IntoPrimitive;
-use num_enum::TryFromPrimitive;
-
 use super::BtfKind;
 use super::BtfType;
 use super::HasSize;
@@ -22,7 +19,7 @@ macro_rules! gen_fieldless_concrete_type {
         $name:ident $(with $trait:ident)?
     ) => {
         $(#[$docs])*
-        #[derive(Debug)]
+        #[derive(Clone, Copy, Debug)]
         pub struct $name<'btf> {
             source: BtfType<'btf>,
         }
@@ -61,7 +58,7 @@ macro_rules! gen_concrete_type {
         $libbpf_ty:ident as $name:ident $(with $trait:ident)?
     ) => {
         $(#[$docs])*
-        #[derive(Debug)]
+        #[derive(Clone, Copy, Debug)]
         pub struct $name<'btf> {
             source: BtfType<'btf>,
             ptr: &'btf libbpf_sys::$libbpf_ty,
@@ -170,7 +167,7 @@ macro_rules! gen_collection_members_concrete_type {
         }
 
         $(#[$docs])*
-        #[derive(Debug)]
+        #[derive(Clone, Copy, Debug)]
         pub struct $member_name $(<$lt>)? {
             $(
                 $(#[$field_docs])*
@@ -193,7 +190,7 @@ macro_rules! gen_collection_concrete_type {
         $($rest:tt)+
     ) => {
         $(#[$docs])*
-        #[derive(Debug)]
+        #[derive(Clone, Copy, Debug)]
         pub struct $name<'btf> {
             source: BtfType<'btf>,
             members: &'btf [libbpf_sys::$libbpf_ty],
@@ -241,7 +238,7 @@ macro_rules! gen_collection_concrete_type {
 }
 
 /// The attributes of a member.
-#[derive(Debug)]
+#[derive(Clone, Copy, Debug)]
 pub enum MemberAttr {
     /// Member is a normal field.
     Normal {
@@ -277,7 +274,7 @@ impl MemberAttr {
 }
 
 /// The kind of linkage a variable of function can have.
-#[derive(TryFromPrimitive, IntoPrimitive, Debug, Clone, Copy, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[repr(u32)]
 pub enum Linkage {
     /// Static linkage
@@ -290,6 +287,25 @@ pub enum Linkage {
     Unknown,
 }
 
+impl From<u32> for Linkage {
+    fn from(value: u32) -> Self {
+        use Linkage::*;
+
+        match value {
+            x if x == Static as u32 => Static,
+            x if x == Global as u32 => Global,
+            x if x == Extern as u32 => Extern,
+            _ => Unknown,
+        }
+    }
+}
+
+impl From<Linkage> for u32 {
+    fn from(value: Linkage) -> Self {
+        value as u32
+    }
+}
+
 impl Display for Linkage {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
@@ -316,7 +332,7 @@ gen_fieldless_concrete_type! {
 /// An integer.
 ///
 /// See also [libbpf docs](https://www.kernel.org/doc/html/latest/bpf/btf.html#btf-kind-int)
-#[derive(Debug)]
+#[derive(Clone, Copy, Debug)]
 pub struct Int<'btf> {
     source: BtfType<'btf>,
     /// The encoding of the number.
@@ -329,7 +345,7 @@ pub struct Int<'btf> {
 }
 
 /// The kinds of ways a btf [Int] can be encoded.
-#[derive(Debug)]
+#[derive(Clone, Copy, Debug)]
 pub enum IntEncoding {
     /// No encoding.
     None,
@@ -453,7 +469,7 @@ gen_collection_concrete_type! {
     /// A member of a [Struct]
     struct StructMember<'btf> {
         /// The member's name
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The member's type
         pub ty: TypeId,
         /// The attributes of this member.
@@ -477,7 +493,7 @@ gen_collection_concrete_type! {
     /// A member of an [Union]
     struct UnionMember<'btf> {
         /// The member's name
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The member's type
         pub ty: TypeId,
         /// The attributes of this member.
@@ -495,7 +511,7 @@ gen_collection_concrete_type! {
 ///
 /// Sometimes it's not useful to distinguish them, in that case, one can use this
 /// type to inspect any of them.
-#[derive(Debug)]
+#[derive(Clone, Copy, Debug)]
 pub struct Composite<'btf> {
     source: BtfType<'btf>,
     /// Whether this type is a struct.
@@ -570,7 +586,7 @@ gen_collection_members_concrete_type! {
     /// A member of a [Struct]
     struct CompositeMember<'btf> {
         /// The member's name
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The member's type
         pub ty: TypeId,
         /// If this member is a bifield, these are it's attributes.
@@ -594,7 +610,7 @@ gen_collection_concrete_type! {
     /// A member of an [Enum]
     struct EnumMember<'btf> {
         /// The name of this enum variant.
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The numeric value of this enum variant.
         pub value: i32,
     }
@@ -679,7 +695,7 @@ impl Func<'_> {
     /// This function's linkage.
     #[inline]
     pub fn linkage(&self) -> Linkage {
-        self.source.vlen().try_into().unwrap_or(Linkage::Unknown)
+        self.source.vlen().into()
     }
 }
 
@@ -693,7 +709,7 @@ gen_collection_concrete_type! {
     /// A parameter of a [FuncProto].
     struct FuncProtoParam<'btf> {
         /// The parameter's name
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The parameter's type
         pub ty: TypeId,
     }
@@ -716,7 +732,7 @@ impl Var<'_> {
     /// The kind of linkage this variable has.
     #[inline]
     pub fn linkage(&self) -> Linkage {
-        self.ptr.linkage.try_into().unwrap_or(Linkage::Unknown)
+        self.ptr.linkage.into()
     }
 }
 
@@ -794,7 +810,7 @@ gen_collection_concrete_type! {
     /// A member of an [Enum64].
     struct Enum64Member<'btf> {
         /// The name of this enum variant.
-        pub name: Option<&'btf CStr>,
+        pub name: Option<&'btf OsStr>,
         /// The numeric value of this enum variant.
         pub value: u64,
     }
@@ -1157,4 +1173,15 @@ mod test {
             }
         });
     }
+
+    #[test]
+    fn linkage_type() {
+        use std::mem::discriminant;
+        use Linkage::*;
+
+        for t in [Static, Global, Extern, Unknown] {
+            // check if discriminants match after a roundtrip conversion
+            assert_eq!(discriminant(&t), discriminant(&Linkage::from(t as u32)));
+        }
+    }
 }
